@@ -42,9 +42,10 @@ import tempfile
 import math
 from shutil import rmtree
 from operator import itemgetter
-from warnings import warn
+import warnings
 from compiler.consts import CO_GENERATOR
 from ConfigParser import ConfigParser
+from itertools import dropwhile
 
 try:
     from test import test_support
@@ -55,14 +56,12 @@ except ImportError:
             pass
     test_support = TestSupport()
 
-from logilab.common.deprecation import class_renamed, deprecated_function, \
-     obsolete
 # pylint: disable-msg=W0622
 from logilab.common.compat import set, enumerate, any, sorted
 # pylint: enable-msg=W0622
 from logilab.common.modutils import load_module_from_name
 from logilab.common.debugger import Debugger, colorize_source
-from logilab.common.decorators import cached
+from logilab.common.decorators import cached, classproperty
 from logilab.common import textutils
 
 
@@ -97,116 +96,6 @@ def with_tempdir(callable):
     return proxy
 
 
-def main(testdir=None, exitafter=True):
-    """Execute a test suite.
-
-    This also parses command-line options and modifies its behaviour
-    accordingly.
-
-    tests -- a list of strings containing test names (optional)
-    testdir -- the directory in which to look for tests (optional)
-
-    Users other than the Python test suite will certainly want to
-    specify testdir; if it's omitted, the directory containing the
-    Python test suite is searched for.
-
-    If the tests argument is omitted, the tests listed on the
-    command-line will be used.  If that's empty, too, then all *.py
-    files beginning with test_ will be used.
-
-    """
-
-    try:
-        opts, args = getopt.getopt(sys.argv[1:], 'hvqxr:t:pcd', ['help'])
-    except getopt.error, msg:
-        print msg
-        print __doc__
-        return 2
-    verbose = 0
-    quiet = False
-    profile = False
-    exclude = []
-    capture = 0
-    for o, a in opts:
-        if o == '-v':
-            verbose += 1
-        elif o == '-q':
-            quiet = True
-            verbose = 0
-        elif o == '-x':
-            exclude.append(a)
-        elif o == '-t':
-            testdir = a
-        elif o == '-p':
-            profile = True
-        elif o == '-c':
-            capture += 1
-        elif o == '-d':
-            global ENABLE_DBC
-            ENABLE_DBC = True
-        elif o in ('-h', '--help'):
-            print __doc__
-            sys.exit(0)
-
-    args = [item.rstrip('.py') for item in args]
-    exclude = [item.rstrip('.py') for item in exclude]
-
-    if testdir is not None:
-        os.chdir(testdir)
-    sys.path.insert(0, '')
-    tests = find_tests('.', args or DEFAULT_PREFIXES, excludes=exclude)
-    # Tell tests to be moderately quiet
-    test_support.verbose = verbose
-    if profile:
-        print >> sys.stderr, '** profiled run'
-        from hotshot import Profile
-        prof = Profile('stones.prof')
-        start_time, start_ctime = time.time(), time.clock()
-        good, bad, skipped, all_result = prof.runcall(run_tests, tests, quiet,
-                                                      verbose, None, capture)
-        end_time, end_ctime = time.time(), time.clock()
-        prof.close()
-    else:
-        start_time, start_ctime = time.time(), time.clock()
-        good, bad, skipped, all_result = run_tests(tests, quiet, verbose, None,
-            capture)
-        end_time, end_ctime = time.time(), time.clock()
-    if not quiet:
-        print '*'*80
-        if all_result:
-            print 'Ran %s test cases in %0.2fs (%0.2fs CPU)' % (
-                all_result.testsRun, end_time - start_time,
-                end_ctime - start_ctime),
-            if all_result.errors:
-                print ', %s errors' % len(all_result.errors),
-            if all_result.failures:
-                print ', %s failed' % len(all_result.failures),
-            if all_result.skipped:
-                print ', %s skipped' % len(all_result.skipped),
-            print
-        if good:
-            if not bad and not skipped and len(good) > 1:
-                print "All",
-            print _count(len(good), "test"), "OK."
-        if bad:
-            print _count(len(bad), "test"), "failed:",
-            print ', '.join(bad)
-        if skipped:
-            print _count(len(skipped), "test"), "skipped:",
-            print ', '.join(['%s (%s)' % (test, msg) for test, msg in skipped])
-    if profile:
-        from hotshot import stats
-        stats = stats.load('stones.prof')
-        stats.sort_stats('time', 'calls')
-        stats.print_stats(30)
-    if exitafter:
-        sys.exit(len(bad) + len(skipped))
-    else:
-        sys.path.pop(0)
-        return len(bad)
-main = obsolete("testlib.main() is obsolete, use the pytest tool instead")(main)
-
-
 def run_tests(tests, quiet, verbose, runner=None, capture=0):
     """Execute a list of tests.
 
@@ -373,6 +262,15 @@ class SkipAwareTestResult(unittest._TextTestResult):
         if self.pdbmode:
             self.debuggers.append(self.pdbclass(sys.exc_info()[2]))
 
+
+    def _iter_valid_frames(self, frames):
+        """only consider non-testlib frames when formatting  traceback"""
+        lgc_testlib = osp.abspath(__file__)
+        std_testlib = osp.abspath(unittest.__file__)
+        invalid = lambda fi: osp.abspath(fi[1]) in (lgc_testlib, std_testlib)
+        for frameinfo in dropwhile(invalid, frames):
+            yield frameinfo
+
     def _exc_info_to_string(self, err, test):
         """Converts a sys.exc_info()-style tuple of values into a string.
 
@@ -384,13 +282,8 @@ class SkipAwareTestResult(unittest._TextTestResult):
         output = ['Traceback (most recent call last)']
         frames = inspect.getinnerframes(tb)
         colorize = self.colorize
-        # count number of relevant levels in the traceback: skip first since
-        # it's our own _proceed function, and then start counting, using
-        # unittest's heuristic
-        nb_frames_skipped = self._count_relevant_tb_levels(tb.tb_next)
-        for index, (frame, filename, lineno, funcname, ctx, ctxindex) in enumerate(frames):
-            if not (0 < index <= nb_frames_skipped):
-                continue
+        frames = enumerate(self._iter_valid_frames(frames))
+        for index, (frame, filename, lineno, funcname, ctx, ctxindex) in frames:
             filename = osp.abspath(filename)
             if ctx is None: # pyc files or C extensions for instance
                 source = '<no source available>'
@@ -462,7 +355,6 @@ class SkipAwareTestResult(unittest._TextTestResult):
 
             self.stream.writeln(self.separator2)
             self.stream.writeln(err)
-
             try:
                 output, errput = test.captured_output()
             except AttributeError:
@@ -497,7 +389,7 @@ def run(self, result, runcondition=None, options=None):
         except TypeError:
             # this might happen if a raw unittest.TestCase is defined
             # and used with python (and not pytest)
-            warn("%s should extend lgc.testlib.TestCase instead of unittest.TestCase"
+            warnings.warn("%s should extend lgc.testlib.TestCase instead of unittest.TestCase"
                  % test)
             test(result)
     return result
@@ -517,7 +409,8 @@ class SkipAwareTextTestRunner(unittest.TextTestRunner):
     def __init__(self, stream=sys.stderr, verbosity=1,
                  exitfirst=False, capture=False, printonly=None,
                  pdbmode=False, cvg=None, test_pattern=None,
-                 skipped_patterns=(), colorize=False, options=None):
+                 skipped_patterns=(), colorize=False, batchmode=False,
+                 options=None):
         super(SkipAwareTextTestRunner, self).__init__(stream=stream,
                                                       verbosity=verbosity)
         self.exitfirst = exitfirst
@@ -528,6 +421,7 @@ class SkipAwareTextTestRunner(unittest.TextTestRunner):
         self.test_pattern = test_pattern
         self.skipped_patterns = skipped_patterns
         self.colorize = colorize
+        self.batchmode = batchmode
         self.options = options
 
     def _this_is_skipped(self, testedname):
@@ -590,35 +484,36 @@ class SkipAwareTextTestRunner(unittest.TextTestRunner):
         stopTime = time.time()
         timeTaken = stopTime - startTime
         result.printErrors()
-        self.stream.writeln(result.separator2)
-        run = result.testsRun
-        self.stream.writeln("Ran %d test%s in %.3fs" %
-                            (run, run != 1 and "s" or "", timeTaken))
-        self.stream.writeln()
-        if not result.wasSuccessful():
-            if self.colorize:
-                self.stream.write(textutils.colorize_ansi("FAILED", color='red'))
+        if not self.batchmode:
+            self.stream.writeln(result.separator2)
+            run = result.testsRun
+            self.stream.writeln("Ran %d test%s in %.3fs" %
+                                (run, run != 1 and "s" or "", timeTaken))
+            self.stream.writeln()
+            if not result.wasSuccessful():
+                if self.colorize:
+                    self.stream.write(textutils.colorize_ansi("FAILED", color='red'))
+                else:
+                    self.stream.write("FAILED")
             else:
-                self.stream.write("FAILED")
-        else:
-            if self.colorize:
-                self.stream.write(textutils.colorize_ansi("OK", color='green'))
-            else:
-                self.stream.write("OK")
-        failed, errored, skipped = map(len, (result.failures, result.errors,
-             result.skipped))
+                if self.colorize:
+                    self.stream.write(textutils.colorize_ansi("OK", color='green'))
+                else:
+                    self.stream.write("OK")
+            failed, errored, skipped = map(len, (result.failures, result.errors,
+                 result.skipped))
 
-        det_results = []
-        for name, value in (("failures", result.failures),
-                            ("errors",result.errors),
-                            ("skipped", result.skipped)):
-            if value:
-                det_results.append("%s=%i" % (name, len(value)))
-        if det_results:
-            self.stream.write(" (")
-            self.stream.write(', '.join(det_results))
-            self.stream.write(")")
-        self.stream.writeln("")
+            det_results = []
+            for name, value in (("failures", result.failures),
+                                ("errors",result.errors),
+                                ("skipped", result.skipped)):
+                if value:
+                    det_results.append("%s=%i" % (name, len(value)))
+            if det_results:
+                self.stream.write(" (")
+                self.stream.write(', '.join(det_results))
+                self.stream.write(")")
+            self.stream.writeln("")
         return result
 
 
@@ -870,6 +765,7 @@ Examples:
                                                   test_pattern=self.test_pattern,
                                                   skipped_patterns=self.skipped_patterns,
                                                   colorize=self.colorize,
+                                                  batchmode=self.batchmode,
                                                   options=self.options)
 
         def removeSucceededTests(obj, succTests):
@@ -913,10 +809,6 @@ succeeded tests into", osp.join(os.getcwd(),FILE_RESTART)
             except Exception, exc:
                 print 'teardown_module error:', exc
                 sys.exit(1)
-        if os.environ.get('PYDEBUG'):
-            warn("PYDEBUG usage is deprecated, use -i / --pdb instead",
-                DeprecationWarning)
-            self.pdbmode = True
         if result.debuggers and self.pdbmode:
             start_interactive_mode(result)
         if not self.batchmode:
@@ -1056,18 +948,6 @@ class InnerTest(tuple):
         instance.name = name
         return instance
 
-class ClassGetProperty(object):
-    """this is a simple property-like class but for
-    class attributes.
-    """
-
-    def __init__(self, getter):
-        self.getter = getter
-
-    def __get__(self, obj, objtype): # pylint: disable-msg=W0613
-        "__get__(objn objtype) -> objtype"
-        return self.getter(objtype)
-
 
 class TestCase(unittest.TestCase):
     """unittest.TestCase with some additional methods"""
@@ -1100,11 +980,12 @@ class TestCase(unittest.TestCase):
         return osp.join(osp.dirname(osp.abspath(mod.__file__)), 'data')
     # cache it (use a class method to cache on class since TestCase is
     # instantiated for each test run)
-    datadir = ClassGetProperty(cached(datadir))
+    datadir = classproperty(cached(datadir))
 
-    def datapath(self, fname):
+    def datapath(cls, fname):
         """joins the object's datadir and `fname`"""
-        return osp.join(self.datadir, fname)
+        return osp.join(cls.datadir, fname)
+    datapath = classmethod(datapath)
 
     def set_description(self, descr):
         """sets the current test's description.
@@ -1130,6 +1011,7 @@ class TestCase(unittest.TestCase):
     def _start_capture(self):
         """start_capture if enable"""
         if self.capture:
+            warnings.simplefilter('ignore', DeprecationWarning)
             self.start_capture()
 
     def _stop_capture(self):
@@ -1209,16 +1091,17 @@ class TestCase(unittest.TestCase):
         try:
             if not self.quiet_run(result, self.setUp):
                 return
+            generative = is_generator(testMethod.im_func)
             # generative tests
-            if is_generator(testMethod.im_func):
-                success = self._proceed_generative(result, testMethod,
-                    runcondition)
+            if generative:
+                self._proceed_generative(result, testMethod,
+                                         runcondition)
             else:
                 status = self._proceed(result, testMethod)
                 success = (status == 0)
             if not self.quiet_run(result, self.tearDown):
                 return
-            if success:
+            if not generative and success:
                 if hasattr(options, "exitfirst") and options.exitfirst:
                     # add this test to restart file
                     try:
@@ -1314,7 +1197,6 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
         """mark a test as skipped for the <msg> reason"""
         msg = msg or 'test was skipped'
         raise TestSkipped(msg)
-    skipped_test = deprecated_function(skip)
 
     def assertIn(self, object, set):
         """assert <object> are in <set>"""
@@ -1376,8 +1258,8 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
 
     def assertSetEquals(self,got,expected, msg=None):
         if not(isinstance(got, set) and isinstance(expected, set)):
-            warn("the assertSetEquals function if now intended for set only."\
-                "use assertUnorderedIterableEquals instead.",
+            warnings.warn("the assertSetEquals function if now intended for set only."\
+                          "use assertUnorderedIterableEquals instead.",
                 DeprecationWarning, 2)
             return self.assertUnorderedIterableEquals(got,expected, msg)
 
@@ -1419,9 +1301,15 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
             self.fail(msg)
     assertListEqual = assertListEquals
 
-    def assertLinesEquals(self, list_1, list_2, msg=None):
+    def assertLinesEquals(self, list_1, list_2, msg=None, striplines=False):
         """assert list of lines are equal"""
-        self.assertListEquals(list_1.splitlines(), list_2.splitlines(), msg)
+        lines1 = list_1.splitlines()
+        if striplines:
+            lines1 = [l.strip() for l in lines1]
+        lines2 = list_2.splitlines()
+        if striplines:
+            lines2 = [l.strip() for l in lines2]
+        self.assertListEquals(lines1, lines2, msg)
     assertLineEqual = assertLinesEquals
 
     def assertXMLWellFormed(self, stream, msg=None):
@@ -1434,19 +1322,12 @@ succeeded test into", osp.join(os.getcwd(),FILE_RESTART)
             if msg is None:
                 msg = 'XML stream not well formed'
             self.fail(msg)
-    assertXMLValid = deprecated_function(assertXMLWellFormed,
-         'assertXMLValid renamed to more precise assertXMLWellFormed')
 
     def assertXMLStringWellFormed(self, xml_string, msg=None):
         """asserts the XML string is well-formed (no DTD conformance check)"""
         stream = StringIO(xml_string)
         self.assertXMLWellFormed(stream, msg)
 
-    assertXMLStringValid = deprecated_function(
-        assertXMLStringWellFormed,
-        'assertXMLStringValid renamed to more precise assertXMLStringWellFormed'
-        )
-
     def assertXMLEqualsTuple(self, element, tup):
         """compare an ElementTree Element to a tuple formatted as follow:
         (tagname, [attrib[, children[, text[, tail]]]])"""
@@ -1796,7 +1677,6 @@ class MockConnection:
         """Mock close method"""
         pass
 
-MockConnexion = class_renamed('MockConnexion', MockConnection)
 
 def mock_object(**params):
     """creates an object using params to set attributes
